using System;
using System.Data;

namespace Akadia.DataRowEdit
{
	// Shows DataRow Editing
	public class DataRowEdit
	{
		public DataRowEdit()
		{
			ShowDataRowEdit();
		}

		public void ShowDataRowEdit()
		{
			// Create the DateTable
			DataTable tabTest = new DataTable("TestTable");

			// Crate a Column
			DataColumn colTest = new DataColumn("colTest",Type.GetType("System.Int32"));

			// Hook our own RowChanged Event Handler to the DataTable
			tabTest.RowChanged+=new DataRowChangeEventHandler(myRowChangedHandler);

			// Add the Column to the DataTable
			tabTest.Columns.Add(colTest);

			// Add a UniqueConstraint to the Column in the DataTable
			tabTest.Constraints.Add(new UniqueConstraint(colTest));

			// Add five Rows
			DataRow rowTest; 
			Console.WriteLine("\n*** Add five Rows to the DataTable\n");
			for(int i=0; i<5; i++)
			{
				// RowChanged Event will occur for every addition!
				rowTest = tabTest.NewRow();
				rowTest["colTest"]= i;
				tabTest.Rows.Add(rowTest);
			}

			// Accept initial Values
			tabTest.AcceptChanges();
			Console.WriteLine("\n*** Accept initial Values");

			// Invoke BeginEdit on each Row
			Console.WriteLine("\n*** Begin Edit on each Row, show RowVersion on each\n");
			foreach(DataRow myRow in tabTest.Rows)
			{    
				myRow.BeginEdit();

				// Add 10 to the inital Values
				// RowChanged Event will occur for every change!
				myRow["colTest"] = (int) myRow["colTest"] + 10;

				// Show RowVersion
				Console.Write("\t Original \t" + myRow["colTest",DataRowVersion.Original]);
				Console.Write("\t Proposed \t" + myRow["colTest",DataRowVersion.Proposed] + "\n");
			}

			// Accept the Changes
			Console.WriteLine("\n");
			tabTest.AcceptChanges();
			Console.WriteLine("\n*** Accept the Changes()\n");

			// Change two Rows to identical values after invoking BeginEdit.
			tabTest.Rows[0].BeginEdit();           // Begin Edit on First Row
			tabTest.Rows[1].BeginEdit();           // Begin Edit on Second Row
			tabTest.Rows[0]["colTest"]= 100;       // Change first Row
			tabTest.Rows[1]["colTest"]= 100;       // Violate Unique Constraint

			try
			{
				// Now invoke EndEdit, this will cause the 
				// UniqueConstraint to be enforced
				tabTest.Rows[0].EndEdit();
				tabTest.Rows[1].EndEdit();
			}
			catch(Exception e)
			{
				// Process Exception, add Message to System Log and return
				System.Diagnostics.EventLog log = new System.Diagnostics.EventLog();
				log.Source = "ShowDataRowEdit";
				log.WriteEntry(e.ToString());
				Console.WriteLine("\n*** Exception of type {0} occurred.", e.GetType());
			}
		}

		// Process RowChanged Event
		private void myRowChangedHandler(object sender, System.Data.DataRowChangeEventArgs e)
		{
			DataTable t = (DataTable)  sender;
			Console.WriteLine("RowChanged: " + e.Action.ToString() + "\t" + e.Row.ItemArray[0]);
		}

		// The main entry point for the application.
		[STAThread]
		static void Main(string[] args)
		{
			DataRowEdit  dataRowEdit;
			dataRowEdit = new DataRowEdit();
		}
	}
}
